@@ -93,7 +93,7 @@ int handle_tick(void *data, int tick)
  * @endcode
  */
 typedef int (*handle_midi_tick_func_t)(void *data, int tick);
-/* @} */
+/** @} */
 
 /**
  * @defgroup midi_events MIDI Events
@@ -114,20 +114,20 @@ FLUIDSYNTH_API void delete_fluid_midi_event(fluid_midi_event_t *event);
 /** @endlifecycle */
 
 FLUIDSYNTH_API int fluid_midi_event_set_type(fluid_midi_event_t *evt, int type);
-FLUIDSYNTH_API int fluid_midi_event_get_type(fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_type(const fluid_midi_event_t *evt);
 FLUIDSYNTH_API int fluid_midi_event_set_channel(fluid_midi_event_t *evt, int chan);
-FLUIDSYNTH_API int fluid_midi_event_get_channel(fluid_midi_event_t *evt);
-FLUIDSYNTH_API int fluid_midi_event_get_key(fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_channel(const fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_key(const fluid_midi_event_t *evt);
 FLUIDSYNTH_API int fluid_midi_event_set_key(fluid_midi_event_t *evt, int key);
-FLUIDSYNTH_API int fluid_midi_event_get_velocity(fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_velocity(const fluid_midi_event_t *evt);
 FLUIDSYNTH_API int fluid_midi_event_set_velocity(fluid_midi_event_t *evt, int vel);
-FLUIDSYNTH_API int fluid_midi_event_get_control(fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_control(const fluid_midi_event_t *evt);
 FLUIDSYNTH_API int fluid_midi_event_set_control(fluid_midi_event_t *evt, int ctrl);
-FLUIDSYNTH_API int fluid_midi_event_get_value(fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_value(const fluid_midi_event_t *evt);
 FLUIDSYNTH_API int fluid_midi_event_set_value(fluid_midi_event_t *evt, int val);
-FLUIDSYNTH_API int fluid_midi_event_get_program(fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_program(const fluid_midi_event_t *evt);
 FLUIDSYNTH_API int fluid_midi_event_set_program(fluid_midi_event_t *evt, int val);
-FLUIDSYNTH_API int fluid_midi_event_get_pitch(fluid_midi_event_t *evt);
+FLUIDSYNTH_API int fluid_midi_event_get_pitch(const fluid_midi_event_t *evt);
 FLUIDSYNTH_API int fluid_midi_event_set_pitch(fluid_midi_event_t *evt, int val);
 FLUIDSYNTH_API int fluid_midi_event_set_sysex(fluid_midi_event_t *evt, void *data,
         int size, int dynamic);
@@ -139,7 +139,7 @@ FLUIDSYNTH_API int fluid_midi_event_set_lyrics(fluid_midi_event_t *evt,
         void *data, int size, int dynamic);
 FLUIDSYNTH_API int fluid_midi_event_get_lyrics(fluid_midi_event_t *evt,
         void **data, int *size);
-/* @} */
+/** @} */
 
 /**
  * @defgroup midi_router MIDI Router
@@ -196,7 +196,7 @@ FLUIDSYNTH_API void fluid_midi_router_rule_set_param2(fluid_midi_router_rule_t *
 FLUIDSYNTH_API int fluid_midi_router_handle_midi_event(void *data, fluid_midi_event_t *event);
 FLUIDSYNTH_API int fluid_midi_dump_prerouter(void *data, fluid_midi_event_t *event);
 FLUIDSYNTH_API int fluid_midi_dump_postrouter(void *data, fluid_midi_event_t *event);
-/* @} */
+/** @} */
 
 /**
  * @defgroup midi_driver MIDI Driver
@@ -227,7 +227,7 @@ fluid_midi_driver_t *new_fluid_midi_driver(fluid_settings_t *settings,
 FLUIDSYNTH_API void delete_fluid_midi_driver(fluid_midi_driver_t *driver);
 /** @endlifecycle */
 
-/* @} */
+/** @} */
 
 /**
  * @defgroup midi_player MIDI File Player
@@ -285,7 +285,7 @@ FLUIDSYNTH_API int fluid_player_get_total_ticks(fluid_player_t *player);
 FLUIDSYNTH_API int fluid_player_get_bpm(fluid_player_t *player);
 FLUIDSYNTH_API int fluid_player_get_midi_tempo(fluid_player_t *player);
 FLUIDSYNTH_API int fluid_player_seek(fluid_player_t *player, int ticks);
-/* @} */
+/** @} */
 
 #ifdef __cplusplus
 }
